/***************************************************************************************
 *	File :	  cxml.h
 *	Company : PDADevelopers.com
 *	Author :  Diego Belfer 
 *	Last Revision : 28/09/2003 (dd/mm/yyyy)
 *	Comments : CPP Wrapper classes
 *             Change these wrappers to your needs
 ***************************************************************************************/

#ifndef _INC_CXML
#define _INC_CXML

#include "IXML.h"

class CXMLParser;
class CXMLNode;



class CXMLNodeIterator {
protected:
	char*		m_name;
	XMLNode		m_node;
public:
	CXMLNodeIterator (XMLNode firstNode, const char *name) : m_node (firstNode) {
		m_name =  (char*)malloc(strlen(name)+1);
		strcpy(m_name, name);
	}
	
	~CXMLNodeIterator() {
		if (m_name)
			delete m_name;
	}

	bool		hasNext()			{ return m_node != NULL; };
	CXMLNode	next();	
	CXMLNode	current();

};


class CXMLNode {
private:
	XMLNode		m_node;
	CXMLNode(XMLNode node) : m_node(node) {};
	friend class CXMLParser;
	friend class CXMLNodeIterator;

public:
	CXMLNode() : m_node( NULL ) {}
	CXMLNode(const CXMLNode& node) : m_node( node.m_node ) {}
	~CXMLNode()  {} ;

	BOOL		isNull()			{ return m_node == NULL;};
	BOOL		hasChildNodes()		{ return xml_hasChildNodes(m_node); };
	uint		childCount()		{ return xml_childCount(m_node); };

	CXMLNode	parentNode()		{ return CXMLNode(xml_parentNode(m_node));	};
	CXMLNode	firstChild()		{ return CXMLNode(xml_firstChild(m_node));	};
	CXMLNode	lastChild()			{ return CXMLNode(xml_lastChild(m_node));	};
	CXMLNode	nextSibling()		{ return CXMLNode(xml_nextSibling(m_node)); };
	CXMLNode	firstAttribute()	{ return CXMLNode(xml_firstAttribute(m_node)); };
	CXMLNode	lastAttribute()		{ return CXMLNode(xml_lastAttribute(m_node)); };

	CXMLNode&	operator=(const CXMLNode& node) { m_node = node.m_node ; return *this;};

	uint		nodeType()							{ return xml_nodeType(m_node);	};
	char*		nodeName(char *buffer, uint size)	{ return xml_nodeName(m_node, buffer, size); };
	char*		nodeValue(char *buffer, uint size)	{ return xml_nodeValue(m_node, buffer, size);};
	char*		text(char *buffer, uint size)		{ return xml_text(m_node, buffer, size);	 };
	
	CXMLNode	getAttributeNode(const char* name)	{ return xml_getAttributeNode(m_node, name); };
	
	char*		getAttribute(const char*name, char *buffer, uint size) { 
		return xml_getAttribute(m_node, name, buffer, size); 
	};
	
	CXMLNodeIterator	getElementsByTagName(const char*name) {
		return CXMLNodeIterator( xml_getElementsByTagName(m_node, name), name); 
	};

	CXMLNode	clone(BOOL deep) 
		{	return CXMLNode( xml_cloneNode(m_node, deep) ); }
	
	/*'const' modifier added due to compiler errors of MW, values of the node are changed**/
	CXMLNode	appendChild(const CXMLNode& node) 
		{	return xml_appendChild(m_node, node.m_node);	}

	/*'const' modifier added due to compiler errors of MW, values of the node are changed*/
	CXMLNode	appendAttribute(const CXMLNode& attr)
		{	return xml_appendAttribute(m_node, attr.m_node);	}

	CXMLNode	insertBefore(CXMLNode& newNode, CXMLNode& before) 
		{	return xml_insertBefore(newNode.m_node, before.m_node);	}

		
	CXMLNode	remove(CXMLNode& node) 
		{	return xml_remove(node.m_node);	}

	CXMLNode	replace(CXMLNode& newNode, CXMLNode& oldNode)
		{	return xml_replace(newNode.m_node, oldNode.m_node);	}

};



class CXMLParser {
private:
	HXML	m_hxml;

public:
	CXMLParser() 
		{	m_hxml = xml_create();	}

	~CXMLParser() 
		{	xml_release(m_hxml);	}

	char*	getDocument()			
		{	return xml_getDocument(m_hxml);						}

	BOOL	processDocument(char *XMLDocument) 
		{	return xml_processDocument(m_hxml, XMLDocument);	}

	CXMLNode getRoot()			
		{	return CXMLNode(xml_getRoot(m_hxml));				}


	uint		getErrorID()		
		{ return xml_getErrorID(m_hxml);  };

	char*		getErrorMsg()		
		{ return xml_getErrorMsg(m_hxml); };

	uint		getErrorPos()		
		{ return xml_getErrorPos(m_hxml); };


	/*Editing Functions*/
	CXMLNode	createRoot()		
		{ return CXMLNode( xml_createRoot(m_hxml)); } 

	CXMLNode	createAttribute(const char *name, const char *value = "") 
		{ return CXMLNode( xml_createAttribute(m_hxml, name, value) ); } 

	CXMLNode	createCDATASection(const char* data = "")	
		{ return CXMLNode( xml_createCDATASection(m_hxml, data) ); } 

	CXMLNode	createTextNode(const char *text = "")	
		{ return CXMLNode( xml_createTextNode(m_hxml, text) ); } 

	CXMLNode	createComment(const char *comment = "")	
		{ return CXMLNode( xml_createComment(m_hxml, comment) ); } 

	CXMLNode	createDocumentFragment()	
		{ return CXMLNode( xml_createDocumentFragment(m_hxml) ); } 

	CXMLNode	createElement(const char *tagName)	
		{ return CXMLNode( xml_createElement(m_hxml, tagName) ); } 

	CXMLNode	createProcessingInstruction(const char *target = "xml", const char *data="version=\"1.0\"")	
		{ return CXMLNode( xml_createProcessingInstruction(m_hxml, target, data ) ); } 

	uint		toXML(char *buffer, uint size) 
		{	return xml_toXML(m_hxml, buffer, size); 	}
};



inline CXMLNode CXMLNodeIterator::next() {
	XMLNode currentNode = m_node;
	m_node = xml_getElementsByTagName_next(m_node, m_name);
	return CXMLNode(currentNode); 
};

inline CXMLNode CXMLNodeIterator::current() {
	return CXMLNode(m_node); 
};

#endif
